# pylint: disable=too-many-branches
# pylint: disable=unused-import
# pylint: disable=line-too-long
import ftputil
import utils.dir as directory
# import utils.file as file
from utils import file


def ftp_connect(address, user_name=None, password=None):
    '''
        Connects to an FTP server.

        ----------

        Arguments
        -------------------------
        `address` {str|dict}
            The FTP address or a dictionary containing the address, user_name and password keys.\n
            If a dictionary is provided, you do not need to give the user_name or password.
        [`user_name`=None] {str}
            The user name used to log in to the FTP server.
        [`password`=None] {str}
            The password used to log in to the FTP server.

        Return {None|object}
        ----------------------
        The ftputil object if successful, None otherwise.

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 12-19-2021 11:28:26
        `memberOf`: colemen_file_utils
        `version`: 1.0
        `method_name`: ftp_connect
    '''
    creds = None
    result = None
    if isinstance(address, (dict)):
        creds = address
    if isinstance(address, (list, tuple)):
        if len(address) == 3:
            creds = {
                "address": address[0],
                "user_name": user_name[1],
                "password": password[2]
            }
    if address is not None and user_name is not None and password is not None:
        creds = {
            "address": address,
            "user_name": user_name,
            "password": password
        }

    if validate_ftp_credentials(creds):
        result = ftputil.FTPHost(creds['address'], creds['user_name'], creds['password'])
        result.synchronize_times()

    return result


def validate_ftp_credentials(obj):
    '''
        Confirms that the dict provided contains the address, user_name and password./n
        It validates that all values are strings and are at least one character in length.
        ----------

        Arguments
        -------------------------
        `obj` {dict}
            A dictionary containing address, user_name and password keys.


        Return {bool}
        ----------------------
        True upon success, false otherwise.

        Meta
        ----------
        `author`: Colemen Atwood
        `created`: 12-19-2021 11:22:03
        `memberOf`: colemen_file_utils
        `version`: 1.0
        `method_name`: validate_ftp_credentials
    '''
    validated = []
    success = False
    if isinstance(obj, (dict)):
        if 'address' in obj:
            if isinstance(obj['address'], (str)):
                if len(obj['address']) > 0:
                    validated.append(obj['address'])
                else:
                    print(f"Invalid address provided: {obj['address']}. Address must be at least one character.")
            else:
                print(f"Invalid address type provided: {type(obj['address'])}. address must be a string.")
        else:
            print("FTP address was not provided.")
        if 'user_name' in obj:
            if isinstance(obj['user_name'], (str)):
                if len(obj['user_name']) > 0:
                    validated.append(obj['user_name'])
                else:
                    print(f"Invalid user_name provided: {obj['user_name']}. user_name must be at least one character.")
            else:
                print(f"Invalid user_name type provided: {type(obj['user_name'])}. user_name must be a string.")
        else:
            print("FTP user_name was not provided.")
        if 'password' in obj:
            if isinstance(obj['password'], (str)):
                if len(obj['password']) > 0:
                    validated.append(obj['password'])
                else:
                    print(f"Invalid password provided: {obj['password']}. password must be at least one character.")
            else:
                print(f"Invalid password type provided: {type(obj['password'])}. password must be a string.")
        else:
            print("FTP password was not provided.")
    else:
        print(f"obj provided must be dictionary, {type(obj)} provided.")
    if len(validated) == 3:
        success = True
    success = False

    return success



# pylint: disable=line-too-long
'''
    Misc. resources of data.
'''
DEST_PATH_SYNONYMS = ["dest", "dst", "dest path", "destination", "destination path", "dst path", "target", "target path"]
SRC_PATH_SYNONYMS = ["source", "src", "src path", "source path", "file path"]



import json
# import shutil
import os
import re
# from pathlib import Path
import utils.objectUtils as objUtils
import utils.file_read as read
import utils.string_utils as strUtils

# todo - DOCUMENTATION FOR METHODS


def by_name(file_name, search_path=None, **kwargs):
    '''
        Searches the path provided for files that match the file_name

        ----------
        Arguments
        -----------------
        `file_name` {str|list}
            The string or list of strings to search for.
        `search_path`=cwd {str}
            The directory to search within.

        Keyword Arguments
        -----------------
            `recursive`=True {boolean}
                If True the path is searched recursively

            `case_sensitive`=True {bool}
                If False case is ignored.
            `exact_match`=True {bool}
                If False it will match with any file that contains the file_name argument
            `regex`=False {bool}
                If True the file_name arg is treated as a regex string for comparisons.

        Return
        ----------
        `return` {None|list}
            A list of matching files or None if no matching files are found.
    '''
    if isinstance(file_name, list) is False:
        file_name = [file_name]
    if search_path is None:
        search_path = os.getcwd()

    extensions = strUtils.format_extension(objUtils.get_kwarg(['extensions', 'ext'], [], (list, str), **kwargs))
    if isinstance(extensions, str):
        extensions = [extensions]

    recursive = objUtils.get_kwarg(['recursive', 'recurse'], True, bool, **kwargs)
    case_sensitive = objUtils.get_kwarg(['case_sensitive'], True, (bool), **kwargs)
    exact_match = objUtils.get_kwarg(['exact_match'], True, (bool), **kwargs)
    regex = objUtils.get_kwarg(['regex', 'use regex'], False, (bool), **kwargs)

    if case_sensitive is False and regex is False:
        new_name_array = []
        for name in file_name:
            if isinstance(name, str):
                new_name_array.append(name.lower())
        file_name = new_name_array

    result_array = []
    # pylint: disable=unused-variable
    # pylint: disable=too-many-nested-blocks
    for root, folders, files in os.walk(search_path):
        for file in files:
            skip = False
            current_file_path = os.path.join(root, file)
            test_file_name = file

            test_file_ext = strUtils.format_extension(os.path.splitext(os.path.basename(current_file_path))[1])
            if case_sensitive is False:
                test_file_name = test_file_name.lower()

            if len(extensions) > 0:
                if test_file_ext not in extensions:
                    skip = True

            if skip is False:
                for name in file_name:
                    if len(name) == 0 or "*" == name:
                        result_array.append(current_file_path)
                        break

                    if regex is not False:
                        match = re.search(name, test_file_name)
                        if match is not None:
                            result_array.append(current_file_path)

                    if exact_match is True:
                        if test_file_name == name:
                            result_array.append(current_file_path)
                    else:
                        if name in test_file_name:
                            result_array.append(current_file_path)
        if recursive is False:
            break

    if len(result_array) > 0:
        return result_array
    return None
